package alun.gchap;

import alun.linkage.LinkageDataSet;
import alun.linkage.LinkageLocus;
import alun.linkage.LinkageIndividual;
import alun.linkage.NumberedAllelePhenotype;

/**
 Represents an observation for a marker locus.
*/
public class MarkerObservation extends Observation
{
/** 
 Creates a new marker observation by reading the data from the given
 linkage data.
 The data for the ith locus in the list is read.
*/
	public MarkerObservation(LinkageDataSet l, int i)
	{
		LinkageLocus loc = l.getParameterData().getLoci()[i];		
		if (loc.nAlleles() == 4 && loc.firstComment().toUpperCase().indexOf("SNP") > -1)
			m = new SNP();
		else
			m = new Marker(loc.nAlleles());

		LinkageIndividual[] ind = l.getPedigreeData().getIndividuals();
		d = new Phenotype[ind.length];

		for (int j=0; j<d.length; j++)
		{
			NumberedAllelePhenotype q = (NumberedAllelePhenotype)(ind[j].getPhenotypes()[i]);
			d[j] = ((Marker)m).findPhenotype(q.a1-1,q.a2-1);
			d[j].incFrequency(1);
		}

		m.downCode();
		m.geneCount(1000);
		m.downCode(10E-10);
	}
}
