package alun.linkage;

/**
 This represents the data for an individual at a locus
 specified in the linkage affected status format.
*/
public class AffectedStatusPhenotype extends LinkagePhenotype
{
/**
 Creates a new phenotype from the given status and liability class numbers.
 The data is checked for consistency with the data for the specified locus.
*/
	public AffectedStatusPhenotype(LinkageAffectationLocus l, int stat, int liab)
	{
		setLocus(l);

		switch(status)
		{
		case UNKNOWN:
		case UNAFFECTED:
		case AFFECTED:
			status = stat;
			break;
		default:
			throw new LinkageException("Inappropriate affectation status "+status);
		}

		if (liab < 1 && liab > l.getLiabilities().length)
			throw new LinkageException("Liability class out of range "+liab);
		else
			liability = liab;
	}

/**
 Returns a string representation of the data.
*/
	public String toString()
	{
		 return f.format(status,2)+" "+f.format(liability,2);
	}

// Private data.

	private static final int UNKNOWN = 0;
	private static final int AFFECTED = 2;
	private static final int UNAFFECTED = 1;

	private int status = 0;
	private int liability = 0;
}
