package alun.linkage;

import alun.util.InputFormatter;
import java.io.IOException;

/**
 This class represents a locus as specified in the
 linkage affectation status format.
*/
public class LinkageAffectationLocus extends LinkageLocus
{
/**
 Creates a new locus with data read from the given input formatter.
*/
	public LinkageAffectationLocus(InputFormatter b) throws IOException
	{
		type = LinkageLocus.AFFECTATION_STATUS;

		freq = new double[b.nextInt()];
		line1comment = b.restOfLine();

		b.newLine();
		for (int i=0; i<freq.length; i++)
			freq[i] = b.nextDouble();
		line2comment = b.restOfLine();

		b.newLine();
		liab = new double[b.nextInt()][];
		comment = new String[liab.length];
		line3comment = b.restOfLine();
		
		int ng = freq.length;
		ng = (ng*(ng+1))/2;
		for (int i=0; i<liab.length; i++)
		{
			b.newLine();
			liab[i] = new double[ng];
			for (int j=0; j<liab[i].length; j++)
				liab[i][j] = b.nextDouble();
			comment[i] = b.restOfLine();
		}
	}

/**
 Reads the data for a phenotype at this locus from the given input
 formatter.
*/
	public LinkagePhenotype readPhenotype(InputFormatter f)
	{
		return new AffectedStatusPhenotype(this, f.nextInt(), f.nextInt());
	}

/**
 Returns the two dimentional array containing the liability data.
*/
	public double[][] getLiabilities()
	{
		return liab;
	}
	
/**
 The two dimentional array containing the liability data.
 These will typically be penetrance probabilities.
*/
	public double[][] liab = null;
/**
 An array holding the comment strings for each linkage liability class.
*/
	public String comment[] = null;
/**
 The string of comment from the second line of input.
*/
	public String line2comment = null;
/**
 The string of comment from the third line of input.
*/
	public String line3comment = null;

/**
 Returns a string representing the parameters of this locus in the
 same format at in the linkage .par file.
*/
	public String toString()
	{
		StringBuffer s = new StringBuffer();

		s.append(type+" "+freq.length+" "+line1comment+"\n");

		for (int i=0; i<freq.length; i++)
			s.append(freq[i]+" ");
		s.append(line2comment+"\n");

		s.append(liab.length+" "+line3comment+"\n");

		for (int i=0; i<liab.length; i++)
		{
			for (int j=0; j<liab[i].length; j++)
				s.append(liab[i][j]+" ");
			s.append(comment[i]+"\n");
		}

		return s.toString();
	}
}
