package alun.linkage;

import alun.util.InputFormatter;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.IOException;

/**
 This class holds the information from a standard Linkage parameter data file
 and a standard Linkage pedigree file.
*/
public class LinkageDataSet
{
/**
 Creates a new LinkageDatSet from the parameter data and the pedigree data
 in the files whose names are given in the constructor.
*/
	public LinkageDataSet(String datain, String pedin) throws IOException
	{
		parfile = datain;
		FileInputStream is = new FileInputStream(parfile);
		InputFormatter f = new InputFormatter(is);
		par = new LinkageParameterData(f);
		is.close();
		
		pedfile = pedin;
		is  = new FileInputStream(pedfile);
		f = new InputFormatter(is);
		ped = new LinkagePedigreeData(f,par);
		is.close();
	}

	public LinkageDataSet(InputStream parin, InputStream pedin) throws IOException
	{
		par = new LinkageParameterData(new InputFormatter(parin));
		ped = new LinkagePedigreeData(new InputFormatter(pedin),par);
	}

/**
 Returns the object containting the data from the linkage .par file.
*/
	public LinkageParameterData getParameterData()
	{
		return par;
	}

/**
 Returns the object containing the data from the linkage .ped file.
*/
	public LinkagePedigreeData getPedigreeData()
	{
		return ped;
	}

/**
 Returns a string representation of the data contained in the data set.
*/
	public String toString()
	{
		return parfile + ":\n" + par + "\n" + pedfile + ":\n" + ped;
	}

// Private data.

	private String parfile = null;
	private LinkageParameterData par = null;
	private String pedfile = null;
	private LinkagePedigreeData ped = null;

/**
 Main reads a Linkage data file and a Linkage pedigree file and writes
 the output to standard output.
*/
	public static void main(String[] args)
	{
		try
		{
			LinkageDataSet L = null;

			switch(args.length)
			{
			case 2:
				L = new LinkageDataSet(args[0],args[1]);
				System.out.println(L);
				break;
			default:
				System.err.println("Specify Linkage parameter file and Linkage pedigree file.");
				System.exit(0);
			}
		}
		catch (Exception e)
		{
			System.err.println("Caught in LinkageDatSet:main()");
			e.printStackTrace();
		}
		
	}
}
