package alun.linkage;

import alun.util.StringFormatter;
import alun.util.InputFormatter;

/**
 A class that contains the information for an individual from the
 linkage .ped file.
*/
public class LinkageIndividual
{
/**
 Creates a new linkage individual by reading the data from the given 
 input formatter. Checks the data for consiency with with the data
 from the parameter file as specified in the second argument.
*/ 
	public LinkageIndividual(InputFormatter b, LinkageParameterData par)
	{
		
		pedid = b.nextInt();
		id = b.nextInt();
		paid = b.nextInt();
		maid = b.nextInt();
		firstbornid = b.nextInt();
		pasnextid = b.nextInt();
		masnextid = b.nextInt();
		sex = b.nextInt();
		proband = b.nextInt();

		LinkageLocus[] l = par.getLoci();
		pheno = new LinkagePhenotype[l.length];
		for (int i=0; i<pheno.length; i++)
			pheno[i] = l[i].readPhenotype(b);

		comment = b.restOfLine();
	}

/**
 Returns the array of linkage phonetypes for this individual.
*/
	public LinkagePhenotype[] getPhenotypes()
	{
		return pheno;
	}

/**
 The individual's pedigree id number.
*/
	public int pedid = 0;
/**
 The individual's id number.
*/
	public int id = 0;
/**
 The individual's father's id number.
*/
	public int paid = 0;
/**
 The individual's  mother's id number.
*/
	public int maid = 0;
/**
 The individual's first born child's id number.
*/
	public int firstbornid = 0;
/**
 The individual's father's next born child's id number.
*/
	public int pasnextid = 0;
/**
 The individual's mother's next born child's id number.
*/
	public int masnextid = 0;
/**
 The individual's sex coded as 1 for male and 2 for female.
*/
	public int sex = 0;
/**
 The individual's proband status coded as 1 for proband 0 for non-proband.
*/
	public int proband = 0;
/**
 A string representing anything that was left on this individual's input line
 after all thee pedigree, sex, proband and locus informatio was read in.
*/
	public String comment = null;
/**
 A list of phenotypes as read in from the input line.
 For co-dominant markers these phenotypes will be the genotypes.
*/
	public LinkagePhenotype[] pheno = null;

/**
 Returns a string representation of this individual's data
 in the same format as the original linkage input .ped file.
*/
	public String toString()
	{
		StringFormatter f = new StringFormatter();
		StringBuffer s = new StringBuffer();

		s.append(f.format(pedid,2)+" ");
		s.append(" ");
		s.append(f.format(id,3)+" ");
		s.append(f.format(paid,3)+" ");
		s.append(f.format(maid,3)+" ");
		s.append(f.format(firstbornid,3)+" ");
		s.append(f.format(pasnextid,3)+" ");
		s.append(f.format(masnextid,3)+" ");
		s.append(f.format(sex,1)+" ");
		s.append(f.format(proband,1)+" ");
		s.append("     ");
		for (int i=0; i<pheno.length; i++)
			s.append(pheno[i]+" ");
		s.append("  ");
		s.append(comment);

		return s.toString();
	} 
}
