package alun.linkage;

import alun.util.InputFormatter;

/**
 This is the base class from which the linkage locus types are
 derived.
 Only the numbered alleles type of locus is currently properly 
 implemented.
*/
abstract public class LinkageLocus 
{
/**
 The type of the locus specified. This will be one of
QUANTITATIVE_VARIABLE, AFFECTATION_STATUS BINARY_FACTORS or NUMBERED_ALLELES.
*/
	public int type = 0;

/**
 Represents the linkage quantiative variable locus type.
*/
	public final static int QUANTITATIVE_VARIABLE = 0;
/**
 Represents the linakge affectation status locus type.
*/
	public final static int AFFECTATION_STATUS = 1;
/**
 Represents the linkage binary factors locus type.
*/
	public final static int BINARY_FACTORS = 2;
/**
 Represents the linkage numbered alleles locus type.
*/
	public final static int NUMBERED_ALLELES = 3;

/**
 A method to read in the data for this locus from the given
 input formatter. This is defined by the subclasses.
*/
	abstract public LinkagePhenotype readPhenotype(InputFormatter f);

/**
 Returns the number of alleles that this locus has.
*/
	public int nAlleles()
	{
		return freq.length;
	}

/**
 Returns the allele frequencies for the locus.
*/
	public double[] alleleFrequencies()
	{
		return freq;
	}

/**
 Returns the comment string from the first line of input
 associated with this locus.
*/
	public String firstComment()
	{
		return line1comment;
	}

// Protected data.

	protected double[] freq = null;
	protected String line1comment = null;
}
