package alun.linkage;

import alun.util.InputFormatter;
import java.io.IOException;

/**
 This class represents the a locus given
 in the linkage numbered locus format.
*/
public class LinkageNumberedLocus extends LinkageLocus
{
/**
 Creates a new locus from data read from the given input formatter.
*/
	public LinkageNumberedLocus(InputFormatter b) throws IOException
	{
		type = NUMBERED_ALLELES;
		int na = b.nextInt();
		line1comment = b.restOfLine();
		
		freq = new double[na];
		b.newLine();
		for (int i=0; i<freq.length; i++)
			freq[i] = b.nextDouble();
		line2comment = b.restOfLine();
	}

/**
 Reads the phenotypic data in the format associated with this
 type of locus from the given input formatter.
*/
	public LinkagePhenotype readPhenotype(InputFormatter f)
	{
		return new NumberedAllelePhenotype(this,f.nextInt(),f.nextInt());
	}


/**
 Returns a String representing the data for this locus in the
 same format as given in the linkage .par file.
*/
	public String toString()
	{
		StringBuffer s = new StringBuffer();
		
		s.append(type+" "+freq.length+" "+line1comment+"\n");
		
		for (int i=0; i<freq.length; i++)
		{
			s.append(freq[i]);
			s.append(" ");
		}
		s.append(line2comment+"\n");

		return s.toString();
	}

// Private data.

	private String line2comment;
}
