package alun.linkage;

import alun.util.InputFormatter;
import java.io.FileInputStream;
import java.io.IOException;

/**
 This structure hold the information associated with the data
 from a linkage parameter file.
*/
public class LinkageParameterData
{
/**
 Creates a new parameter file structure using data read from 
 the given input formatter.
*/
	public LinkageParameterData(InputFormatter b) throws IOException
	{
		b.newLine();
		nloci = b.nextInt();
		risklocus = b.nextInt();
		sexlinked = b.nextInt();
		programcode = b.nextInt();
		line1comment = b.restOfLine();

		b.newLine();
		mutlocus = b.nextInt();
		mutmale = b.nextDouble();
		mutfemale = b.nextDouble();
		disequilibrium = b.nextInt();
		line2comment = b.restOfLine();

		b.newLine();
		order = new int[nloci];
		for (int i=0; i<order.length; i++)
			order[i] = b.nextInt();
		line3comment = b.restOfLine();

		locus = new LinkageLocus[nloci];
		for (int i=0; i<locus.length; i++)
		{
			b.newLine();
			int type = b.nextInt();
			switch(type)
			{
			case LinkageLocus.AFFECTATION_STATUS:
				locus[i] = new LinkageAffectationLocus(b);
				break;
			case LinkageLocus.NUMBERED_ALLELES:
				locus[i] = new LinkageNumberedLocus(b);
				break;
			case LinkageLocus.QUANTITATIVE_VARIABLE:
				//locus[i] = new LinkageQuantitativeLocus(b);
			case LinkageLocus.BINARY_FACTORS:
				//locus[i] = new LinkageBinaryLocus(b);
			default:
				break;
			}
		}

		b.newLine();
		sexdifference = b.nextInt();
		interference = b.nextInt();
		line4comment = b.restOfLine();
		
		b.newLine();
		distances = new double[nloci-1];
		for (int j = 0; j<distances.length; j++)
			distances[j] = b.nextDouble();
		line5comment = b.restOfLine();

		b.newLine();
		variablelocus = b.nextInt();
		increment = b.nextDouble();
		stoppingvalue = b.nextDouble();
		line6comment = b.restOfLine();
	}

/**
 Returns the array of loci in this set of data.
*/
	public LinkageLocus[] getLoci()
	{
		return locus;
	}

/**
 Returns the number of loci in this set of data.
*/
	public int nLoci()
	{
		return locus.length;
	}

/**
 The number of loci in this set of data.
*/
	public int nloci = 0;
/**
 The index of the risk locus in the list of loci.
*/
	public int risklocus = 0;
	public int sexlinked = 0;
	public int programcode = 0;
	public String line1comment = null;

	public int mutlocus = 0;
	public double mutmale = 0;
	public double mutfemale = 0;
	public int disequilibrium = 0;
	public String line2comment = null;

	public int[] order = null;
	public String line3comment = null;

	public LinkageLocus[] locus = null;

	public int sexdifference = 0;
	public int interference = 0;
	public String line4comment = null;

	public double[] distances = null;
	public String  line5comment = null;

	public int variablelocus = 0;
	public double increment = 0;
	public double stoppingvalue = 0; 
	public String line6comment = null;

/**
 Returns a string representing the data for a collection of 
 loci in the same format as in the .par file from which it
 was read.
*/
	public String toString()
	{
		StringBuffer s = new StringBuffer();

		s.append(nloci+" "+risklocus+" "+sexlinked+" "+programcode+" "+line1comment+"\n");

		s.append(mutlocus+" "+mutmale+" "+mutfemale+" "+disequilibrium+" "+line2comment+"\n");

		for (int i=0; i<order.length; i++)
			s.append(order[i]+" ");
		s.append(line3comment+"\n");

		for (int i=0; i<nloci; i++)
			s.append(locus[i]);

		s.append(sexdifference+" "+interference+" "+line4comment+"\n");
		
		for (int i=0; i<distances.length; i++)
			s.append(distances[i]+" ");
		s.append(line5comment+"\n");

		s.append(variablelocus+" "+increment+" "+stoppingvalue+" "+line6comment+"\n");

		s.setLength(s.length()-1);
		return s.toString();
	}

/**
 Test main.
*/
	public static void main(String[] args)
	{
		try
		{
			String file = null;

			switch(args.length)
			{
				case 1:
					file = args[0];
					break;
				default:
					System.exit(2);
					break;
			}

			FileInputStream is = new FileInputStream(file);
			InputFormatter f = new InputFormatter(is);
			LinkageParameterData l = new LinkageParameterData(f);
			is.close();
			System.out.println(l);
		}
		catch (Exception e)
		{
			System.err.println("Caught in LinkageParameterData:main()");
			e.printStackTrace();
		}
	}
}
