package alun.linkage;

import alun.util.InputFormatter;
import java.util.Vector;
import java.io.FileInputStream;
import java.io.IOException;

/**
 A structure representing the data from a linkage .ped file.
*/
public class LinkagePedigreeData 
{
/**
 Creates a new data structure by reading pedigree and phenotype data from
 the given input formatter. The data is checked for consistency with the
 parameters in the given parameter data structure.
*/
	public LinkagePedigreeData(InputFormatter b, LinkageParameterData par) throws IOException
	{
		Vector v = new Vector();

		while (b.newLine())
		{
			LinkageIndividual i = new LinkageIndividual(b,par);
			v.addElement(i);
		}

		ind = (LinkageIndividual[])v.toArray(new LinkageIndividual[v.size()]);
	}
	
/**
 Returns the array of individual data structures.
*/
	public LinkageIndividual[] getIndividuals()
	{
		return ind;
	}

/**
 Returns a string representing the data for the given pedigrees in
 the same format as it was read in from the linkage .ped file.
*/
	public String toString()
	{
		StringBuffer s = new StringBuffer();
		for (int i=0; i<ind.length; i++)
			s.append(ind[i]+"\n");
		return s.toString();
	}

// Private data.

	private LinkageIndividual[] ind = null;

/**
 Test main.
*/
	public static void main(String[] args)
	{
		try
		{
			String file = null;

			switch(args.length)
			{
			case 1:
				file = args[0];
				break;
			default:
				System.exit(0);
			}

			FileInputStream is = new FileInputStream(file);
			InputFormatter f = new InputFormatter(is);
			LinkagePedigreeData l = new LinkagePedigreeData(f,null);
			is.close();
			System.out.print(l);
		}
		catch (Exception e)
		{
			System.err.println("Caught in LinkagePedigreeData:main()");
			e.printStackTrace();
		}
	}
}
