package alun.util;

import java.util.StringTokenizer;
import java.io.Reader;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.IOException;

/**
 This class lets the programmer read input in a formatted way.
*/
public class InputFormatter
{
/** 
 Creates a new input formatter by opening a file with the given file name
 using the characters in the other string as field delimitors.
 If the given file name is null, the standard input stream is used instead.
 If the delimitor string is null, the built in defaults of StringTokenizer are used. 

 As an example the following will access all the integers in a 
 file of unknown length and find the sum.

	int total = 0;
	InputFormatter f = new InputFormatter();
	while (f.newLine())
	{
		while (f.newToken())
		{
			total += f.getInt();
		}
	}
	
*/

	public InputFormatter(Reader is, String del) throws IOException
	{
		b = new BufferedReader(is);
		delim = del;
	}

	public InputFormatter(Reader is) throws IOException
	{
		this(is,null);
	}

	public InputFormatter(InputStream is, String del) throws IOException
	{
		this (new InputStreamReader(is),del);
	}

	public InputFormatter(InputStream is) throws IOException
	{
		this(new InputStreamReader(is));
	}

	public InputFormatter() throws IOException
	{
		this(System.in);
	}

/**
 Skips the remainder of the current input line. If there is 
 another line it returns true, otherwise false.
*/
	public boolean newLine() throws IOException
	{
		String s = b.readLine();
		if (s == null)
			return false;
		if (delim == null)
			t = new StringTokenizer(s);
		else
			t = new StringTokenizer(s,delim);
		return true;	
	}

/**
 Skips to the next token in the current line. If there is
 another token it returns true, otherwise false.
*/
	public boolean newToken() 
	{
		if (t.hasMoreTokens())
		{
			tok = t.nextToken();
			return true;
		}
		else
		{
			return false;
		}
	}

/**
 Returns the number of items remaining on the current line.
*/
	public int itemsLeftOnLine()
	{
		return t.countTokens();
	}

/**
 Returns the value of the current token as an int.
*/
	public int getInt() 
	{
		return (new Integer(tok)).intValue();
	}
		
/**
 Returns the value of the current token as a double.
*/
	public double getDouble()
	{
		return (new Double(tok)).doubleValue();
	}

/**
 Returns the value of the current token as a String.
*/
	public String getString()
	{
		return tok;
	}

/**
 Advances to the next token and returns it as an integer.
 No checks are made to ensure that this makes sense to do.
*/
	public int nextInt()
	{
		return (new Integer(t.nextToken())).intValue();
	}

/**
 Advances to the next token and returns it as a double.
 No checks are made to ensure that this makes sense to do.
*/
	public double nextDouble()
	{
		return (new Double(t.nextToken())).doubleValue();
	}

	public String nextString()
	{
		return t.nextToken();
	}
	
/**
 Returns the remainder of the current line as a string and
 skips to the end of the line. Does not move on to the next line.
*/
	public String restOfLine()
	{
		if (t.hasMoreTokens())
			return t.nextToken("\n");
		else
			return "";
	}
		
// Priate data.

	private BufferedReader b = null;
	private StringTokenizer t = null;
	private String tok = null;
	private String delim = null;
}
