#!c:/perl/bin/perl.exe

if ($#ARGV != 1) {
    printf OUT "Useage: $0 infile outfile\n";
} else {

    #
    # @genotypes stores the name of genotypes (ids) in the input.
    # @homolog1 stores the first sequence for an genotype.
    # @homolog2 stores the second sequence for an genotype.
    # %known_haps stores homozygote sequences.
    #
    my($infile, $outfile, $i, $j, $index, $first_line, $curr_genotype, $count);
    my($num_samples, $homo_seq, $num_homozygotes, $total_homozygotes); 
    my(%genotypes, %homolog1, %homolog2, %known_haps, %haplotypes);
    my(%original, $is_match, $complement, $homozygotes, $known_hap);
    my($hap_seq1, $hap_seq2, $key2, @used_keys, %hap_count, %hap_members);
    my(%adjusted_hap_freqs, %hap_freqs, $hap_total, @expected_geno_count);
    my(@phenotpyes, @phenotypes_count, $num_phenos, @phenos_count);
    my($pheno_i, $pheno_j, %save_hap_pairs, $total_haps, $num_haps);
    my(@skip_sites, $temp_sequence, $length, $temp_base, $rand_value, $geno);

    $infile = $ARGV[0];
    $outfile = $ARGV[1];
    open(IN, $infile) ;
    open(OUT, ">$outfile") ;

print STDERR $outfile."\n";

    #
    # Skip sites allows specific sites in the sequence to be skipped.
    #
    @skip_sites = ();
#    @skip_sites = (1,6,8,9,11,14,15,16,18,19,22,26,27,34,50,51,55,56,57,61,63,64,65,67,69,74);

    $count = 0;
    #loop through input file store in appropriate hash.
    while (<IN>) {
	if (/^\s*(\#\S*)\s*$/) {
	    $curr_genotype = $1;
	    $genotypes{$count} = $curr_genotype;
	    $original{$count} = $curr_genotype;
	    $first_line = 1;
	} elsif ((/\s*([acgtnACGTN]+)\s*/i) && $first_line) {
	    $temp_sequence = $1;
	    $temp_sequence =~ tr/A-Z/a-z/;
	    $homolog1{$curr_genotype} = $temp_sequence;
	    $first_line = 0;
	} elsif ((/\s*([acgtnACGTN]+)\s*/i)) {
	    $temp_sequence = $1;
	    $temp_sequence =~ tr/A-Z/a-z/;
	    $homolog2{$curr_genotype} = $temp_sequence;
	    $count++;
	}
    } #while input

    $num_samples = $count;

    if ($skip_sites[0]) {
	for ($i = 0; $i < @skip_sites; $i++) {
	    foreach $geno (values(%genotypes)) {
		$homolog1{$geno} = 
		    &blank_position($homolog1{$geno}, $skip_sites[$i]);
		$homolog2{$geno} = 
		    &blank_position($homolog2{$geno}, $skip_sites[$i]);
	    }
	}
    }	    
	
    #Randomize genotypes
    foreach $geno (values(%genotypes)) {
	$length = length($homolog1{$geno});
	for ($i = 0; $i < $length; $i++) {
	    if (rand(1) > 0.5) {
		($homolog1{$geno}, $homolog2{$geno}) =
		    &swap_position($homolog1{$geno}, $homolog2{$geno}, $i);
	    }
	}
    } #for all genotypes    

    #output randomized genotypes
    &output_genotypes(\%genotypes, \%homolog1, \%homolog2);

    #determine haplotypes
#    (%haplotypes, $num_haplotypes) =
#	&get_haplotypes(\%genotypes, \%homolog1, \%homolog2);
#
#    foreach $hap (keys(%haplotypes)) {
#	print OUT "$haplotypes{$hap} $hap\n";
#    }

    $num_haps = 1;
    #Homozygotes Section
    print OUT "\nHomozygotes:\n";
    $num_homozygotes = 0;
    $total_homozygotes = 0;
    $homozygotes = "";
    foreach $geno (values(%genotypes)) {
	#test for homozygosity
	if (&equal_sequences($homolog1{$geno}, $homolog2{$geno})) {
	    print OUT "$geno\n";
	    $total_homozygotes++;
	    $homozygotes = $homozygotes.", ". $geno;
	    $homo_seq = $homolog1{$geno};

	    #is it a known homozygote?
	    if (&is_member($homo_seq, %known_haps)) {
		$key = &get_key($homo_seq, %known_haps);
		print OUT "$homolog1{$geno} $key\n";
		print OUT "$homolog2{$geno} $key\n";
		$hap_count{$key} = $hap_count{$key} + 2;
		$hap_members{$key} = $hap_members{$key}.", ".$geno;
		$key1 = $key2 = $key;
	    } else {
		print OUT "$homolog1{$geno} $num_haps\n";
		print OUT "$homolog2{$geno} $num_haps\n";
		$hap_count{$num_haps} = 2;
		$hap_members{$num_haps} = "$geno";
		$num_homozygotes++;
		$known_haps{$num_haps} = $homo_seq;
		$key1 = $key2 = $num_haps;
		$num_haps++;
	    }
	    $save_hap_pairs{$geno} = "($key1, $key2)";
	    foreach $key (keys(%genotypes)) {
		if ($genotypes{$key} eq $geno) {
		    delete $genotypes{$key};
		}
	    }
	    delete $homolog1{$geno};
	    delete $homolog2{$geno};
	}
    } # for all genotypes 
    
    printf OUT "homozygotes : $homozygotes\n";
    &output_haplotypes(\%known_haps, \%hap_count, \%hap_members);
    printf OUT "The number of homozygotes = $total_homozygotes\n";
    printf OUT "The number of distinct homozygotes = $num_homozygotes\n";

    #Single-site heterozygotes
    printf OUT "\nSingle site hetrozygotes:\n";
    $total_sshetro = 0;
    foreach $geno (values(%genotypes)) {
	#test for off by one
	if (&off_by_one($homolog1{$geno}, $homolog2{$geno})) {
	    printf OUT "$geno\n";
	    $total_sshetro++;

	    #is homolog1 a known haplotype
	    if (&is_member($homolog1{$geno}, %known_haps)) {
		$key = &get_key($homolog1{$geno}, %known_haps);
		printf OUT "$homolog1{$geno} $key\n";
		$hap_count{$key} = $hap_count{$key} + 1;
		$hap_members{$key} = $hap_members{$key}.", ". $geno;
		$key1 = $key;
	    } else {
		printf OUT "$homolog1{$geno} $num_haps\n";
		$hap_count{$num_haps} = 1;
		$hap_members{$num_haps} = "$geno";
		$key1 = $num_haps;
		$known_haps{$num_haps} = $homolog1{$geno};
		$num_haps++;
	    }
	    #is homolog2 a known haplotype
	    if (&is_member($homolog2{$geno}, %known_haps)) {
		$key = &get_key($homolog2{$geno}, %known_haps);
		printf OUT "$homolog2{$geno} $key\n";
		$hap_count{$key} = $hap_count{$key} + 1;
		$hap_members{$key} = $hap_members{$key}.", ". $geno;
		$key2 = $key;
	    } else {
		printf OUT "$homolog2{$geno} $num_haps\n";
		$hap_count{$num_haps} = 1;
		$hap_members{$num_haps} = "$geno";
		$key2 = $num_haps;
		$known_haps{$num_haps} = $homolog2{$geno};
		$num_haps++;
	    }
	    $save_hap_pairs{$geno} = "($key1, $key2)";
	    foreach $key (keys(%genotypes)) {
		if ($genotypes{$key} eq $geno) {
		    delete $genotypes{$key};
		}
	    }
	    delete $homolog1{$geno};
	    delete $homolog2{$geno};
	}
    } # for all genotypes 
    
    printf OUT "The number of single site hetrozygotes = $total_sshetro\n";
    printf OUT "Count of unambiguous haplotypes = ".($num_haps - 1)."\n";

    &short_output_haplotypes(\%known_haps);

    #=========================================================
    #      Now for each of the known haplotypes, check each
    #      of the remaining genotypes to see whether it could
    #      form the given known haplotype.  If so, then check
    #      to see whether the complement is a new haplotype.
    #      If so, then add the complement
    #      haplotype to the known haplotypes, and go back to
    #      the beginning of the known haplotypes.  If you reach
    #      the end of the known haplotypes, then no new haplotypes
    #      were added, so stop.  You might check then at the end
    #      to see whether any of the diploids could form more than
    #      2 haplotypes (>1 if homozygous), indicating the solution
    #      may not be unique.
    #=========================================================


    $pre_num_genos = 0; $post_num_genos = 1;
    while ($pre_num_genos != $post_num_genos) {
#	printf OUT "Start: $pre_num_genos $post_num_genos\n";
	$pre_num_genos = 0;
	foreach $geno (values(%genotypes)) {
	    $pre_num_genos++;
	}
	$count = keys(%known_haps);
	for ($known_hap = 1; $known_hap < $count; $known_hap++) {
#	    printf OUT "known hap = $known_hap\n";
	    foreach $geno (values(%genotypes)) {
#		printf OUT "geno = $geno\n";
		($is_match, $complement) = 
		    &adjust4match($known_haps{$known_hap}, 
				  $homolog1{$geno}, 
				  $homolog2{$geno}, 0);
#		printf OUT "adjusted?: $is_match \"$complement\"\n";
		if ($is_match) {
		    $key1 = $known_hap;
		    printf OUT "$geno\n";
		    printf OUT "$known_haps{$known_hap} $known_hap\n";
		    $hap_count{$known_hap} = $hap_count{$known_hap} + 1;
		    $hap_members{$known_hap} = 
			$hap_members{$known_hap}.", ". $geno;
		    #is complement a known haplotype
		    if (&is_member($complement, %known_haps)) {
			$key = &get_key($complement, %known_haps);
			printf OUT "$complement $key\n";
			$hap_count{$key} = $hap_count{$key} + 1;
			$hap_members{$key} = $hap_members{$key}.", ". $geno;
			$key2 = $key;
		    } else {
			printf OUT "$complement $num_haps\n";
			$hap_count{$num_haps} = 1;
			$hap_members{$num_haps} = "$geno";
			$key2 = $num_haps;
			$known_haps{$num_haps} = $complement;
			$num_haps++;
		    }
		    foreach $key (keys(%genotypes)) {
			if ($genotypes{$key} eq $geno) {
			    delete $genotypes{$key};
			}
		    }
		    $save_hap_pairs{$geno} = "($key1, $key2)";
		    delete $homolog1{$geno};
		    delete $homolog2{$geno};
		}

	    } #for remaining genotypes
	} #for all known haplotypes
	$post_num_genos = 0;
	foreach $geno (values(%genotypes)) {
	    $post_num_genos++;
	}
    } #while still removing genotypes
#    printf OUT "End: $pre_num_genos $post_num_genos\n";

    if ($post_num_genos == 0) {
	printf OUT "All genotypes resolved\n";
    } else {
        #=========================================================
        #      Now for each of the known haplotypes, check each
        #      of the remaining genotypes to see whether it could
        #      form the given known haplotype WITH ONE MUTATION.
        #=========================================================
	printf OUT "One mutation:\n";
	#the following code is the same above loop except call to adjust4match
	#now allows 1 mismatch
	$pre_num_genos = 0; $post_num_genos = 1;
	while ($pre_num_genos != $post_num_genos) {
#	printf OUT "Start: $pre_num_genos $post_num_genos\n";
	    $pre_num_genos = 0;
	    foreach $geno (values(%genotypes)) {
		$pre_num_genos++;
	    }
	    $count = keys(%known_haps);
	    for ($known_hap = 1; $known_hap < $count; $known_hap++) {
#	    printf OUT "known hap = $known_hap\n";
		foreach $geno (values(%genotypes)) {
#		printf OUT "geno = $geno\n";
		    ($is_match, $complement) = 
			&adjust4match($known_haps{$known_hap}, 
				      $homolog1{$geno}, 
				      $homolog2{$geno}, 1);
		    $key1 = $known_hap;
#		printf OUT "adjusted?: $is_match \"$complement\"\n";
		    if ($is_match) {
			printf OUT "$geno\n";
			printf OUT "$known_haps{$known_hap} $known_hap\n";
			$hap_count{$known_hap} = $hap_count{$known_hap} + 1;
			$hap_members{$known_hap} = 
			    $hap_members{$known_hap}.", ". $geno;
			#is complement a known haplotype
			if (&is_member($complement, %known_haps)) {
			    $key = &get_key($complement, %known_haps);
			    printf OUT "$complement $key\n";
			    $hap_count{$key} = $hap_count{$key} + 1;
			    $hap_members{$key} = $hap_members{$key}.", ". $geno;
			    $key2 = $key;
			} else {
			    printf OUT "$complement $num_haps\n";
			    $hap_count{$num_haps} = 1;
			    $hap_members{$num_haps} = "$geno";
			    $key2 = $num_haps;
			    $known_haps{$num_haps} = $complement;
			    $num_haps++;
			}
			foreach $key (keys(%genotypes)) {
			    if ($genotypes{$key} eq $geno) {
				delete $genotypes{$key};
			    }
			}
			$save_hap_pairs{$geno} = "($key1, $key2)";
			delete $homolog1{$geno};
			delete $homolog2{$geno};
		    }

		} #for remaining genotypes
	    } #for all known haplotypes
	    $post_num_genos = 0;
	    foreach $geno (values(%genotypes)) {
		$post_num_genos++;
	    }
	} #while still removing genotypes
#	printf OUT "End: $pre_num_genos $post_num_genos\n";
    }

    printf OUT "\n\#mega\nTITLE: Hapinfer output data\n";
    &short_output_haplotypes(\%known_haps);

    if ($post_num_genos != 0) {
	#output unresolved genotypes
	printf OUT "Unresolved genotypes were:\n";
	&output_genotypes(\%genotypes, \%homolog1, \%homolog2);
        #Now see if they can be constructed as recombinants
	$count = keys(%genotypes);
	for ($i = 0; $i < $count; $i++) {
	    printf OUT "$genotypes{$i}\n$homolog1{$i}\n$homolog2{$i}\n";
	    foreach $key (keys(%known_haps)) {
		printf OUT "$key : ";
		&is_recombinant($known_haps{$key}, 
				$homolog1{$i}, 
				$homolog2{$i});
	    } #for all known haplotypes
	} #for remaining genotypes
    } 

#    &output_haplotypes(\%known_haps, \%hap_count, \%hap_members);
    $num_phenos = 0;
    $count = keys(%original);
    for ($i = 0; $i < $count; $i++) {
	printf OUT "$original{$i}\n";
	@used_keys = ();
	$copy_of_pairs = $save_hap_pairs{$original{$i}};
	while ($copy_of_pairs =~ s/(\(\d*, \d*\))//) {
	    $temp = $1;
	    $temp =~ /\((\d*), (\d*)\)/;
	    $key1 = $1;
	    $key2 = $2;
	    if ($key1 > $key2) {
		$temp = $key1;
		$key1 = $key2;
		$key2 = $temp;
	    }
	    printf OUT "$known_haps{$key1} - $key1\n";
	    printf OUT "$known_haps{$key2} - $key2\n";
	    push(@used_keys, $key1, $key2);
	    $hap_seq1 = $known_haps{$key1};
	    $hap_seq2 = $known_haps{$key2};
	    if ($phenotypes_count[$key1][$key2]) {
		$phenos_count[$key1][$key2] = $phenos_count[$key1][$key2] + 1;
		$phenotypes_count[$key1][$key2] =
		    $phenotypes_count[$key1][$key2] + 1;
	    } else {
		$phenotypes_count[$key1][$key2] = 1;
		$phenos_count[$key1][$key2] = 1;
		$num_phenos++;
		$phenotypes[$num_phenos][0] = $key1;
		$phenotypes[$num_phenos][1] = $key2;
	    }
	}
        # Now loop through all known haplotypes and see if this unphased
        # genotype could be composed of any other pair of known haps.
        # If so, then printf OUT the haplotype pair indented.
	$j = 2;
	$num_haps = keys(%known_haps);
	for ($known_hap = 1; $known_hap < $num_haps; $known_hap++) {
#	    printf OUT "known_hap = $known_hap\n";
	    if (!(&member($known_hap, @used_keys))) {
		($is_match, $complement) = 
		    &adjust4match($known_haps{$known_hap}, $hap_seq1,
				  $hap_seq2, 0); 
		if ($is_match) {
		    #is complement a known haplotype
		    if (&is_member($complement, %known_haps)) {
			$key2 = &get_key($complement, %known_haps);
			push(@used_keys, $known_hap, $key2);
			printf OUT "$known_haps{$known_hap} - $known_hap\n";
			printf OUT "$known_haps{$key2} - $key2\n";
			$save_hap_pairs{$original{$i}} = 
			    $save_hap_pairs{$original{$i}}."($known_hap, $key2)";
			if ($phenotypes_count[$known_hap][$key2]) {
			    $phenotypes_count[$known_hap][$key2] =
				$phenotypes_count[$known_hap][$key2] + 1;
			} else {
			    $phenotypes_count[$known_hap][$key2] = 1;
			    $num_phenos++;
			    $phenotypes[$num_phenos][0] = $known_hap;
			    $phenotypes[$num_phenos][1] = $key2;
			}
		    }
		} #matches a additional known haplotype
	    } #on a different key?
	} # for all keys
    }
    printf OUT "number of phenotypes : $num_phenos\n";
    for ($i = 1; $i <= $num_phenos; $i++) {
	printf OUT "$i : \($phenotypes[$i][0], $phenotypes[$i][1]\) : $phenotypes_count[$phenotypes[$i][0]][$phenotypes[$i][1]]\n";
    }
#    &output_haplotype_freqs(\%known_haps, \%hap_count, $num_samples);
    #Expectation Maximization Step
    foreach $index (keys(%known_haps)) {
	$total_haps = $total_haps + $hap_count{$index};
    } #for all known haplotypes
    foreach $index (keys(%known_haps)) {
	$hap_freqs{$index} = $hap_count{$index} / $total_haps;
	$adjusted_hap_freqs{$index} = $hap_count{$index} / $total_haps;
    } #for all known haplotypes
    for ($em_iteration = 0; $em_iteration < 50; $em_iteration++) {
	#clear counts
	for ($i = 1; $i <= $num_phenos; $i++) {
	    for ($j = 1; $j <= $num_phenos; $j++) {
		$expected_geno_count[$i][$j] = 0;
	    }
	} 

	for ($i = 1; $i <= $num_phenos; $i++) {
	    $total = 0;
	    #first calc what fraction of this pheno is exp to be each geno
	    $pheno_i = $phenotypes[$i][0];
	    $pheno_j = $phenotypes[$i][1];
	    if ($pheno_i > $pheno_j) {
		$temp = $pheno_i;
		$pheno_i = $pheno_j;
		$pheno_j = $temp;
	    }
#	    printf OUT "em $i : \($pheno_i, $pheno_j\) : $phenotypes_count[$pheno_i][$pheno_j] : $adjusted_hap_freqs{$pheno_i} : $adjusted_hap_freqs{$pheno_j}\n";
	    if ($pheno_i == $pheno_j) {
		$total = $total + 
		    ($phenotypes_count[$pheno_i][$pheno_j] * 
		     ($adjusted_hap_freqs{$pheno_i} *
		      $adjusted_hap_freqs{$pheno_j}));
	    } else {
		$total = $total + 
		    ($phenotypes_count[$pheno_i][$pheno_j] * 
		     (2.0 * ($adjusted_hap_freqs{$pheno_i} *
			  $adjusted_hap_freqs{$pheno_j})));
	    }
#	    printf OUT "total : $i $total\n";
	    if ($pheno_i == $pheno_j) {
		if ($total == 0) {
		    $frequency = 0;
		} else {
		    $frequency = 
			$phenotypes_count[$pheno_i][$pheno_j] * 
			 ((($phenos_count[$pheno_i][$pheno_j] * 
			    ($adjusted_hap_freqs{$pheno_i} *
			     $adjusted_hap_freqs{$pheno_i}))) / $total);
		}
		$expected_geno_count[$pheno_i][$pheno_j] =
		    ($phenotypes_count[$pheno_i][$pheno_j] * $frequency);
	    } else {
		if ($total == 0) {
		    $frequency = 0;
		} else {
		    $frequency = ($phenotypes_count[$pheno_i][$pheno_j] * 
				  ((($phenos_count[$pheno_i][$pheno_j] * 
				     (2.0 * ($adjusted_hap_freqs{$pheno_i} *
					     $adjusted_hap_freqs{$pheno_j})))) /
				   $total));
		}
		$expected_geno_count[$pheno_i][$pheno_j] =
		    ($phenotypes_count[$pheno_i][$pheno_j] * $frequency);
	    }	    
	}
	# This completes the expectation step, and you now have a matrix of
	# expected genotype counts.  Now do gene counting to tally up the 
	# frequencies of each haplotype.
	$num_haps = keys(%known_haps);
	for ($i = 1; $i <= $num_haps; $i++) {
	    $tally_hap_freq{$i} = 0;
	}
	for ($i = 1; $i <= $num_haps; $i++) {
	    for ($j = $i; $j <= $num_haps; $j++) {
		$tally_hap_freq{$i} = $tally_hap_freq{$i} + 
		    $expected_geno_count[$i][$j];
		$tally_hap_freq{$j} = $tally_hap_freq{$j} + 
		    $expected_geno_count[$i][$j];
	    }
	}
	$total = 0;
	for ($i = 1; $i <= $num_haps; $i++) {
	    $total = $total + $tally_hap_freq{$i};
	}
	for ($i = 1; $i <= $num_haps; $i++) {
	    $tally_hap_freq{$i} = $tally_hap_freq{$i} / $total;
#	    printf OUT "adjusting $i $adjusted_hap_freqs{$i} -> $tally_hap_freq{$i}\n";
	    $adjusted_hap_freqs{$i} = $tally_hap_freq{$i};
	}
    } #expectation maximization loop
    $num_haps = keys(%known_haps);
    printf OUT "Index Count Freq0 Freq-ML Count-ML\n";
    for ($i = 1; $i <= $num_haps; $i++) {
	printf OUT "%i %3i %5.3f %5.3f %5.3f\n", $i, $hap_count{$i},
	       $hap_freqs{$i}, $adjusted_hap_freqs{$i}, 
	       ($adjusted_hap_freqs{$i} * ($num_samples * 2));
    }
    $count = keys(%original);
    for ($i = 0; $i < $count; $i++) {
	printf OUT "$original{$i}\n";
#	printf OUT "$save_hap_pairs{$original{$i}}\n";
	$copy_of_pairs = $save_hap_pairs{$original{$i}};
	$j = 0;
	$max_freq = 0;
	$max_j = 1;
	while ($copy_of_pairs =~ s/(\(\d*, \d*\))//) {
	    $temp = $1;
	    $temp =~ /\((\d*), (\d*)\)/;
	    $hap1 = $1;
	    $hap2 = $2;
	    if ($hap1 > $hap2) {
		$temp = $hap1;
		$hap1 = $hap2;
		$hap2 = $temp;
	    }
	    if ($hap1 == $hap2) {
		$efreq{$j} = $adjusted_hap_freqs{$hap1} * 
		    $adjusted_hap_freqs{$hap1};
	    } else {
		$efreq{$j} = (2.0 * ($adjusted_hap_freqs{$hap1} * 
				     $adjusted_hap_freqs{$hap2}));
	    }
	    if ($efreq{$j} > $max_freq) {
		$max_freq = $efreq{$j};
		$max_j = $j;
	    }
	    $j++;
	}
	$j = 0;
	$copy_of_pairs = $save_hap_pairs{$original{$i}};
	while ($copy_of_pairs =~ s/(\(\d*, \d*\))//) {
	    $temp = $1;
	    $temp =~ /\((\d*), (\d*)\)/;
	    $hap1 = $1;
	    $hap2 = $2;
	    if ($hap1 > $hap2) {
		$temp = $hap1;
		$hap1 = $hap2;
		$hap2 = $temp;
	    }
	    printf OUT "%s - %i %8.6f\n", 
		   $known_haps{$hap1}, $hap1, ($efreq{$j}/$max_freq);
	    printf OUT "%s - %i\n", $known_haps{$hap2}, $hap2;
	    $j++;
	}
    }
    close(IN);
    print STDERR "Closing out\n";
    close(OUT);
} #if input

#
# Replace position in string with blank char
#
sub blank_position {
    my ($in_string, $position) = @_;

    my ($return_string, $length, $i);

    $length = length($in_string);
    for ($i = 0; $i < $length; $i++) {
	# minus 1 is a fortran conversion thing
	if ($i == ($position - 1)) {
	    $return_string = $return_string." ";
	} else {
	    $return_string = $return_string.substr($in_string, $i, 1);
	}
    }
    return($return_string);
} #blank_position

#
# Swap position of input strings
#
sub swap_position {
    my ($in_string1, $in_string2, $position) = @_;

    my ($return_string1, $return_string2, $length, $i);

    $return_string1 = $return_string2 = "";
    $length = length($in_string1); # assume both the same
    for ($i = 0; $i < $length; $i++) {
	if ($i == $position) {
	    $return_string1 = $return_string1.substr($in_string2, $i, 1);
	    $return_string2 = $return_string2.substr($in_string1, $i, 1);
	} else {
	    $return_string1 = $return_string1.substr($in_string1, $i, 1);
	    $return_string2 = $return_string2.substr($in_string2, $i, 1);
	}
    }
    return($return_string1, $return_string2); 
} #swap_position

#
# Test if two sequences are equivalate.
# 'n' is discounted i.e. counts as a match.
#
sub equal_sequences {
    my ($in_string1, $in_string2) = @_;

    my ($length, $i);

    $length = length($in_string1); # assume both the same
    for ($i = 0; $i < $length; $i++) {
	if ((substr($in_string1, $i, 1) ne substr($in_string2, $i, 1)) &&
	    ((substr($in_string1, $i, 1) ne 'n') ||
	     (substr($in_string2, $i, 1) ne 'n'))) {
	    return(0); #false
	}
    }
    return(1); #true
} #equal_sequences

#
# Test if two sequences are off by 1 base.
# 'n' is discounted i.e. counts as a match.
#
sub off_by_one {
    my ($in_string1, $in_string2) = @_;

    my ($length, $i, $num_mismatches);

    $length = length($in_string1); # assume both the same
    $num_mismatches = 0;
    for ($i = 0; $i < $length; $i++) {
	if ((substr($in_string1, $i, 1) ne substr($in_string2, $i, 1)) &&
	    ((substr($in_string1, $i, 1) ne 'n') ||
	     (substr($in_string2, $i, 1) ne 'n'))) {
	    $num_mismatches++;
	}
    }
    if ($num_mismatches == 1) {
	return(1); #true
    } else {
	return(0); #false
    }
} #off_by_one

#
# Test if two sequences are adjustable to known haplotype sequence
# within number of mismatches allowed
# 'n' is discounted i.e. counts as a match.
#
sub adjust4match {
    my ($hap_string, $in_string1, $in_string2, $mismatches_allowed) = @_;

    my ($length, $i, $return_string, $curr_mismatches);

    $return_string = "";

    $curr_mismatches = 0;
    $length = length($hap_string); # assume both the same
    for ($i = 0; $i < $length; $i++) {
	if ((substr($hap_string, $i, 1) ne substr($in_string1, $i, 1)) &&
	    (substr($hap_string, $i, 1) ne substr($in_string2, $i, 1)) &&
	    ((substr($hap_string, $i, 1) ne 'n') ||
	     (substr($in_string1, $i, 1) ne 'n') ||
	     (substr($in_string2, $i, 1) ne 'n'))) {
	    $curr_mismatches++;
	    if ($curr_mismatches > $mismatches_allowed) {
		return(0, $return_string);
	    }
	} 
	if ((substr($hap_string, $i, 1) ne substr($in_string1, $i, 1)) &&
	    ((substr($hap_string, $i, 1) ne 'n') ||
	     (substr($in_string1, $i, 1) ne 'n'))) {
	    $return_string = $return_string.substr($in_string1, $i, 1);
       	} else {
	    $return_string = $return_string.substr($in_string2, $i, 1);
	}
    }
    return(1,$return_string); #true
} #adjust4match

#
# Run through strings for matches. printf OUT 1 if any match, 0 otherwise.
# 'n' is discounted i.e. counts as a match.
#
sub is_recombinant {
    my ($hap_string, $in_string1, $in_string2) = @_;

    my ($length, $i);

    $length = length($hap_string); # assume both the same
    for ($i = 0; $i < $length; $i++) {
	if ((substr($hap_string, $i, 1) ne substr($in_string1, $i, 1)) &&
	    (substr($hap_string, $i, 1) ne substr($in_string2, $i, 1)) &&
	    ((substr($hap_string, $i, 1) ne 'n') ||
	     (substr($in_string1, $i, 1) ne 'n') ||
	     (substr($in_string2, $i, 1) ne 'n'))) {
	    printf OUT "0";
	} else {
	    printf OUT "1";
	}
    }
    printf OUT "\n";
} #is_recombinant

#
# Test if sequence is member of a hash
#
sub is_member {
    my ($in_sequence, %hash) = @_;

    foreach $key (values(%hash)) {
	if (&equal_sequences($in_sequence, $key)) {
	    return(1); #true
	} 
    }
    return(0); #false
} #is_member

#
# Test if element is part of an array
#
sub member {
    my ($element, @array) = @_;

    my($value);
    foreach $value ( @array ) {
	if ($value == $element) {
	    return(1); #true
	} 
    }
    return(0); #false
} #member

#
# Get key for sequence in a hash
#
sub get_key {
    my ($in_sequence, %hash) = @_;

    foreach $key (keys(%hash)) {
	if (&equal_sequences($in_sequence, $hash{$key})) {
	    return($key); #true
	} 
    }
} #get_key

#
# Output the genotypes for a hash
#
sub output_genotypes {
    local(*list, *homolog1, *homolog2) = @_;

    my($key);
    foreach $key (keys(%list)) {
	printf OUT $list{$key}."\n";
	printf OUT $homolog1{$list{$key}}."\n";
	printf OUT $homolog2{$list{$key}}."\n";
    }
} #output_genotypes

#
# Output the haplotypes for a hash
#
sub output_haplotypes {
    local(*haplist, *hapcount, *hapmembers) = @_;

    my($key);
    printf OUT "\nCurr Haplotypes:\n";
    printf OUT "Index : Sequence : Count : Members\n";
    foreach $key (keys(%haplist)) {
	printf OUT "$key : $haplist{$key} : ";
	printf OUT "$hapcount{$key} : $hapmembers{$key}\n";
    } #for all known haplotypes

} #output_haplotypes

#
# Output the haplotypes for a hash
#
sub output_haplotype_freqs {
    local(*haplist, *hapcount, $sample_count) = @_;

    my($index, $length, $total_haps);

    printf OUT "Index : Sequence : Count : Frequency\n";

    $total_haps = 0;
    foreach $index (keys(%haplist)) {
	$total_haps = $total_haps + $hapcount{$index};
    } #for all known haplotypes
    $length = keys(%haplist);
    for ($index = 1; $index <= $length; $index++) {
	printf("%3i : %s : %3i : %5.3f\n", $index, $haplist{$index},
	       $hapcount{$index}, ($hapcount{$index}/(2 * $sample_count)));
    } #for all known haplotypes

} #output_haplotype_freqs

#
# Output the haplotypes for a hash
#
sub short_output_haplotypes {
    local(*haplist) = @_;

    my($index, $length);
    printf OUT "\nList of Unambiguous Haplotypes:\n";
    $length = keys(%haplist);
    for ($index = 1;$index <= $length; $index++) {
	printf OUT "$index : $haplist{$index}\n";
    } #for all known haplotypes

} #output_haplotypes


#
# Determine the haplotypes.
#
sub get_haplotypes {
    local(*list, *homolog1, *homolog2) = @_;

    my(%return_list, $key, $hap_count);

    $hap_count = 0;
    foreach $key (keys(%list)) {
	if (!(&is_member($homolog1{$key}, %return_list))) {
	    $return_list{$homolog1{$key}} = $hap_count;
	    $hap_count++;
	}
	if (!(&is_member($homolog2{$key}, %return_list))) {
	    $return_list{$homolog2{$key}} = $hap_count;
	    $hap_count++;
	}
    }
    return(%return_list, $hap_count);
} #get_haplotypes


